
/******************************************************************************
                     Definition of Constants
 ******************************************************************************/

#define MAXINT      2147483647

#define success     1
#define fail        0

#define SOURCE      0
#define TARGET      1

#define null        0

#define Rankmax     1                /* Maximum Rank of input array */
#define Nmax        131072              /* Maximum global domain size  */
#define N_Lmax      Nmax/32           /* Maximum local domain size   */
#define Procmax     32              /* Maximum number of processor on each dim*/
#define Tot_Procmax 32               /* Maximum total number of processor */

#define V_Lmax      N_Lmax 
#define Bufmax      N_Lmax 
#define Indexmax    131072

#define TAG         3
#define intsize     4
#define datasize    4
#define msg_size    4                /* integer */

#define abs(x)      ( (x) < 0     ?  -(x)  :  (x) )  
#define max(x,y)    ( (x) < (y)   ?   (y)  :  (x) )
#define min(x,y)    ( (x) < (y)   ?   (x)  :  (y) )

#define Ceiling(j,k)   ( ( (j) + (k) - 1 ) / (k) )
#define Floor(j,k)     ( (j) / (k) )

#define empty       0


/******************************************************************************
                     Definition of Type
 ******************************************************************************/
 
typedef int Datatype; /* data type for array */

typedef int Gindex;   /* for global index */

typedef int Lindex;   /* for local index */

typedef int Pindex;   /* for processor index */

typedef enum {false, true} 
    Boolean;

typedef struct {        /* type for initial packed elements */
  Lindex lidx[Rankmax]; /* local index on each dim */
  int    tnum0;         /* tile number on dim 0 */
  Gindex rank;          /* rank */
  Pindex dest;          /* destination processor */
} Storage_type;


/******************************************************************************
                     Definition of Global variables
 ******************************************************************************/
 
int rank;               /* Rank of input array A */

int N[Rankmax];         /* Global Array size for A & M */

/*--------------------- Processor Information ----------------------*/

int ori_nproc,          /* Number of available  processors */
    tot_nproc,          /* Total number of processors */
    tot_dproc;          /* dimension of tot_nproc */

int nproc[Rankmax],     /* Total number of processors on each dim s*/
    dproc[Rankmax];     /* Total dimension of processors */

int my_pnum,            /* Processor number */
    my_grid[Rankmax],   /* My pnum in grid form */ 
    other_grid[Tot_Procmax][Rankmax];   /* Each  pnums of grid form */


/*------------- Input Distribution Information (A & M) -------------*/

int width[Rankmax],     /* Width of partition within a single tile */
                        /* that is, block size */
    twidth[Rankmax],    /* Width of each tile */
    ntile[Rankmax];     /* Total Number of tiles */

int L[Rankmax];         /* Locall Domain size */

int      my_grid[Rankmax];             /* My pnum in grid form */
int      grid[Tot_Procmax][Rankmax];   /* Each  pnums of grid form */
 
Boolean  s_active;                 

/*---------- Local Input Array (A & M) & Output Vector (V) ---------*/

Datatype A1[N_Lmax];            /* Input Array */
Datatype A2[1][1];

Boolean M1[N_Lmax];             /* Mask Array */
Boolean M2[1][1];

Datatype V[V_Lmax];              /* Result Vector */

int Vwidth;                    /* Block size of V */
int V_L;                        /* Local vector size */

/*------------------------------ PACK ------------------------------*/

int Psize;                /* Number of packed element */
int LPsize;               /* Number of local packed element */

    /*--- Change name (5/9): Conflict with scarab.h ---*/
    /*---                 PS => PS_                 ---*/

    /*--- Dynamic allocation works (7/14) ---*/
/*****
int *PS_[Rankmax],  
    *RS[Rankmax];
*****/

int PS_[Rankmax][Bufmax],   
    RS[Rankmax][Bufmax]; 
  
int PRS_size[Rankmax];        /* Size of Intermediate Array */

Storage_type storage[N_Lmax]; /* Initial packed elements */
                                     /* Can't use dynamic allocation */

/*------------------------- Communication --------------------------*/

int      sendl[Tot_Procmax];     /* Row vector of comm matrix */
Datatype sendB[Bufmax*2];        /* element + rank */
int      send_offset[Tot_Procmax];

int      recvl[Tot_Procmax];     /* Column vector of comm matrix */
Datatype recvB[Bufmax*2];        /* element + rank */
int      recv_offset[Tot_Procmax];

extern int Self, PS;
